// Serpiente con Clase
// (C) 2002 Con Clase http://www.conclase.net
// Salvador Pozo Coronado

// Implementacin de la clase para CJuego:

#include "juego.h"

CJuego::CJuego(HWND hw, HINSTANCE instancia) : hwnd(hw), pausa(true)
{
   HMENU menu, submenu;

   // Crear objetos:
   Graficos = new CGraficos(instancia, hwnd);
   Laberinto = new CLaberinto(Graficos);
   Comida = new CComida(Graficos, Laberinto);
   Extra = new CExtra(Graficos, Laberinto);
   Tanteo = new CTanteo(Graficos);
   Comida->Situar(CCoordenada(16,6));
   Serpiente = new CSerpiente(Graficos, Laberinto, Comida, Extra, Tanteo);
   // Actualizar men de laberintos:
   LeerMenu(hwnd);
   // Crear un array para puntuaciones mximas:
   puntuacion = new int[nLaberintos];
   // Leer puntuaciones mximas y opciones:
   LeerRegistro();
   // Actualizar men:
   menu = GetMenu(hwnd);
   submenu = GetSubMenu(menu, 1);
   CheckMenuItem(GetSubMenu(submenu, 0), laberinto, MF_BYPOSITION | MF_CHECKED);            
   CheckMenuItem(GetSubMenu(submenu, 1), velocidad, MF_BYPOSITION | MF_CHECKED);            
   // Iniciar laberinto y serpiente:
   Laberinto->Iniciar(laberinto);
   Serpiente->Iniciar(laberinto);
}

CJuego::~CJuego()
{
   GuardarRegistro();
   delete Serpiente;
   delete Graficos;
   delete Laberinto;
   delete Comida;
   delete Tanteo;
   delete puntuacion;
}

void CJuego::LeerMenu(HWND hwnd)
{
   char cad[30];
   HMENU menu, submenu;
   ifstream fmenu("laberintos.dat");
   
   fmenu.getline(cad, 30); // Cuenta
   nLaberintos = atoi(cad);
   menu = GetMenu(hwnd);
   submenu = GetSubMenu(menu, 1);
   RemoveMenu(GetSubMenu(submenu, 0), 0, MF_BYPOSITION); // Borrar el actual
   for(int i = 0; i < nLaberintos; i++) {
      fmenu.getline(cad, 30); // Nombre
      AppendMenu(GetSubMenu(submenu, 0), MF_ENABLED | MF_STRING, 100+i, cad);
   }
}

void CJuego::LeerRegistro()
{
   HKEY clave;
   DWORD disposicion;
   DWORD tamanio;
   char contador[32];

   // Crear o abrir la clave de registro:
   RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\ConClase\\Serpiente",
       0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
       &clave, &disposicion);

   if(disposicion == REG_CREATED_NEW_KEY) { // Es necesario crear los valores
      for(int i = 0; i < nLaberintos; i++) {
         wsprintf(contador, "Laberinto%04d", i);
         puntuacion[i] = 0; // Valor inicial a cero.
         RegSetValueEx(clave, contador, 0, REG_DWORD,
            (BYTE *)&puntuacion[i], sizeof(int));
      }
      laberinto = 0;
      RegSetValueEx(clave, "Laberinto", 0, REG_DWORD,
         (BYTE *)&laberinto, sizeof(int));
      velocidad = 2;
      RegSetValueEx(clave, "Velocidad", 0, REG_DWORD,
         (BYTE *)&velocidad, sizeof(int));
   }
   else { // Leer valores desde el registro:
      tamanio = sizeof(int);
      for(int i = 0; i < nLaberintos; i++) {
         wsprintf(contador, "Laberinto%04d", i);
         RegQueryValueEx(clave, contador, NULL, NULL,
            (BYTE*)&puntuacion[i], &tamanio);
      }
      RegQueryValueEx(clave, "Laberinto", NULL, NULL,
         (BYTE*)&laberinto, &tamanio);
      RegQueryValueEx(clave, "Velocidad", NULL, NULL,
         (BYTE*)&velocidad, &tamanio);
   }
   RegCloseKey(clave);
}

void CJuego::GuardarRegistro()
{
   HKEY clave;
   DWORD disposicion;
   DWORD tamanio;
   char contador[32];

   // Crear o abrir la clave de registro:
   RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\ConClase\\Serpiente",
       0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
       &clave, &disposicion);

   for(int i = 0; i < nLaberintos; i++) {
      wsprintf(contador, "Laberinto%04d", i);
      RegSetValueEx(clave, contador, 0, REG_DWORD,
         (BYTE *)&puntuacion[i], sizeof(DWORD));
   }
   RegSetValueEx(clave, "Laberinto", 0, REG_DWORD,
      (BYTE *)&laberinto, sizeof(DWORD));
   RegSetValueEx(clave, "Velocidad", 0, REG_DWORD,
      (BYTE *)&velocidad, sizeof(DWORD));
   RegCloseKey(clave);
}

void CJuego::Mostrar(HDC dc)
{
   Serpiente->Mostrar(dc);
   Comida->Mostrar(dc);
   Laberinto->Mostrar(dc);
   Tanteo->Mostrar(dc);
   Graficos->Texto(CCoordenada(13,ALTO), puntuacion[laberinto]);
}

void CJuego::CambiaLaberinto(int valor)
{
   HMENU menu, submenu;

   menu = GetMenu(hwnd);
   submenu = GetSubMenu(menu, 1);
   
   CheckMenuItem(GetSubMenu(submenu, 0), laberinto, MF_BYPOSITION | MF_UNCHECKED);
   laberinto = valor;
   CheckMenuItem(GetSubMenu(submenu, 0), laberinto, MF_BYPOSITION | MF_CHECKED);            
   Laberinto->Iniciar(laberinto);
   Serpiente->Iniciar(laberinto);
   Graficos->Borrar();
}

void CJuego::CambiaVelocidad(int valor)
{
   HMENU menu, submenu;

   menu = GetMenu(hwnd);
   submenu = GetSubMenu(menu, 1);

   CheckMenuItem(GetSubMenu(submenu, 1), velocidad, MF_BYPOSITION | MF_UNCHECKED);
   velocidad = valor;
   CheckMenuItem(GetSubMenu(submenu, 1), velocidad, MF_BYPOSITION | MF_CHECKED);   
}

void CJuego::ActivarMenus(bool activar)
{
   HMENU menu, submenu;
   
   menu = GetMenu(hwnd);
   submenu = GetSubMenu(menu, 1);
   if(activar) {
      EnableMenuItem(submenu, 0, MF_BYPOSITION | MF_ENABLED);
      EnableMenuItem(submenu, 1, MF_BYPOSITION | MF_ENABLED);
   }
   else {
      EnableMenuItem(submenu, 0, MF_BYPOSITION | MF_GRAYED);
      EnableMenuItem(submenu, 1, MF_BYPOSITION | MF_GRAYED);
   }
}

void CJuego::Movimiento()
{
   if(!Serpiente->Avanzar()) {
      // Detener juego:
      pausa = true;
      KillTimer(hwnd, 1); 
      // Actualizar puntuacin mxima:
      if(Tanteo->Puntos() > puntuacion[laberinto])
         puntuacion[laberinto] = Tanteo->Puntos();
      // Enable menus:
      ActivarMenus(true);
   }
   if(Extra->Activo()) Extra->Siguiente();
   else Extra->Situar();
}

void CJuego::Empezar()
{
   if(pausa) {
      // Iniciar objetos
      Laberinto->Iniciar(laberinto);
      Serpiente->Iniciar(laberinto);
      Tanteo->Iniciar();
      Graficos->Borrar();
      Extra->Anular();
      SetTimer(hwnd, 1, 300-50*velocidad, NULL);
      pausa = false;
      // Disable menus:
      ActivarMenus(false);
   }
}

void CJuego::MoverSerpiente(eDireccion dir)
{
   Serpiente->Deseada(dir);
}

