// Fichero de programa para Serpiente con Clase
// (C) 2002 Con Clase http://www.conclase.net
// Salvador Pozo Coronado

#include <windows.h>
#include <fstream>
#include "ids.h"
#include "juego.h"

using namespace std;

CJuego *Juego;

/*  Procedimientos de ventana y dilogo  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DlgSobreProc(HWND, UINT, WPARAM, LPARAM);

/*  Nombre de la clase de ventana  */
char szNombreClase[ ] = "SerpienteApp";

int WINAPI
WinMain (HINSTANCE hInstance,
         HINSTANCE hPrevInstance,
         LPSTR lpszArgument,
         int nFunsterStil)

{
    HWND hwnd;               /* El manipulador de nuestra ventana */
    MSG messages;            /* Mensajes que nuestra aplicacin manipular */
    WNDCLASSEX wincl;        /* Estructura de datos para windowclass */
    RECT ventana, cliente;
    
    /* The Window structure */
    wincl.hInstance = hInstance;
    wincl.lpszClassName = szNombreClase;
    wincl.lpfnWndProc = WindowProcedure;      /* Procedimiento de ventana */
    wincl.style = CS_DBLCLKS;                 /* Trata los double-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar el icono que hemos definico */
    wincl.hIcon = LoadIcon (hInstance, "Icono");
    wincl.hIconSm = LoadIcon (hInstance, "Icono");
    /* Y el cursor por defecto */
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";  /* Men */
    wincl.cbClsExtra = 0;         /* No hay informacin extra para la clase */
    wincl.cbWndExtra = 0;         /* estructura o instancia de ventana */
    /* Color de fondo para la ventana */
    wincl.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);

    /* Registrar la clase de ventana, si falla salir del programa */
    if (!RegisterClassEx (&wincl)) return 0;

    /* La clase ha sido registrada, creamos el programa */
    hwnd = CreateWindowEx (
           0,                       /* Posibilidades aadidas */
           szNombreClase,           /* Nombre de la clase */
           "Serpiente con Clase",   /* Ttulo */
           WS_SYSMENU | WS_MINIMIZEBOX,     /* Tipo de ventana */
           CW_USEDEFAULT,           /* Windows decide la posicin */
           CW_USEDEFAULT,           /* de la ventana en pantalla */
           100,                     /* Anchura (no importa) */
           100,                     /* Altura  (no importa) */
           HWND_DESKTOP,            /* Ventana hija del escritorio */
           NULL,                    /* menu de la clase */
           hInstance,               /* Manipulador de la instancia */
           NULL                     /* No hay datos de creacin de ventana */
           );

    /* Calcular las dimensiones de la ventana en funcin del rea de cliente
       que queremos tener */
    GetWindowRect(hwnd, &ventana);
    GetClientRect(hwnd, &cliente);
    ventana.right  = ventana.right-ventana.left-cliente.right+cliente.left+ANCHO*16;
    ventana.bottom = ventana.bottom-ventana.top-cliente.bottom+cliente.top+(ALTO+1)*16;
    MoveWindow(hwnd, ventana.left, ventana.top, ventana.right, ventana.bottom, FALSE);

    /* Mostrar la ventana */
    ShowWindow (hwnd, nFunsterStil);

    /* Ejecutar el bucle de mensajes */
    while (TRUE == GetMessage (&messages, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de carcter */
        TranslateMessage(&messages);
        /* Enviar el mensaje al procedimiento de ventana */
        DispatchMessage(&messages);
    }

    /* El valor de retorno es el dado por PostQuitMessage() */
    return messages.wParam;
}


/*  This function is called by the Windows function DispatchMessage()  */

LRESULT CALLBACK
WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   PAINTSTRUCT ps;
   HDC dc;
   static HINSTANCE instancia;
   
   switch (message)                  /* handle the messages */
   {
      case WM_CREATE:
         instancia = ((LPCREATESTRUCT)lParam)->hInstance;
         Juego = new CJuego(hwnd, instancia);
         break;
      case WM_PAINT:
         dc = BeginPaint(hwnd, &ps); 
         Juego->Mostrar(dc);
         EndPaint(hwnd, &ps);
         break;
      case WM_COMMAND:
         switch(LOWORD(wParam)) {
            case CM_SALIR:
               DestroyWindow(hwnd);
               break;
            case CM_SOBRE:
               DialogBox(instancia, "DialogoSobre", hwnd, DlgSobreProc); 
               break;
            default:
                if(LOWORD(wParam) >= 100 && LOWORD(wParam) < 200)  // laberinto
                   Juego->CambiaLaberinto(LOWORD(wParam)-100);
                else  // Velocidad
                   Juego->CambiaVelocidad(LOWORD(wParam)-200);
         }
         break;
      case WM_KILLFOCUS:
         if(!Juego->Parado()) KillTimer(hwnd,1); 
         break;
      case WM_SETFOCUS:
         if(!Juego->Parado()) SetTimer(hwnd, 1, 300-50*Juego->Velocidad(), NULL);
         break;
      case WM_TIMER:
         if(wParam == 1) {
            Juego->Movimiento();
            return 0;
         }        
         break;
      case WM_KEYDOWN:
         switch((int)wParam) {
            case VK_LEFT:  Juego->MoverSerpiente(izquierda); break;
            case VK_UP:    Juego->MoverSerpiente(arriba); break;
            case VK_RIGHT: Juego->MoverSerpiente(derecha); break;
            case VK_DOWN:  Juego->MoverSerpiente(abajo); break;
            // Si el juego est parado, debe empezar
            default:       Juego->Empezar();
         }
         break;
      case WM_DESTROY:
         KillTimer(hwnd, 1);
         delete Juego;
         PostQuitMessage(0);       /* send a WM_QUIT to the message queue */
         break;
      default:                      /* for messages that we don't deal with */
         return DefWindowProc (hwnd, message, wParam, lParam);
   }
   return 0;
}

BOOL CALLBACK DlgSobreProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    if(msg == WM_COMMAND)                  /* manipulador del mensaje */
    {
       EndDialog(hDlg, FALSE);
       return TRUE;
    }
    return FALSE;
}

