// Serpiente con Clase
// (C) 2002 Con Clase http://www.conclase.net
// Salvador Pozo Coronado

// Implementacin de la clase para CSerpiente:

#include <stdlib.h>
#include "serpient.h"

CSerpiente::CSerpiente(CGraficos *graf, CLaberinto *lab, 
   CComida *com, CExtra *ext, CTanteo *tan) : 
   ColaSecciones(), direccion(derecha),
   deseada(derecha), porCrecer(0), Comida(com), Extra(ext),
   Graficos(graf), Laberinto(lab), Tanteo(tan)
{
   Iniciar(0);
}

void CSerpiente::Iniciar(int n)
{ 
   CSeccion seccion;
   int i, cuenta;
   ifstream datos;
   char cad[20]; 
   char *szX, *szY, *szTipo;
   
   // Destruir serpiente:
   ColaSecciones.Vaciar();

   wsprintf(cad, "%04d.lab", n);
   datos.open(cad); 
         
   datos.getline(cad, 20); // Nombre
   datos.getline(cad, 20); // Cuenta laberinto
   cuenta = atoi(cad);
   for(i = 0; i < cuenta; i++) 
      datos.getline(cad, 20); // ignorar

   datos.getline(cad, 20); // Nombre
   datos.getline(cad, 20); // Cuenta serpiente
   cuenta = atoi(cad);

   for(i = 0; i < cuenta; i++) {
      datos.getline(cad, 20);
      szX = strtok(cad, ",");
      szY = strtok(NULL, ",");
      szTipo = strtok(NULL, "\n");
      seccion = CSeccion(CCoordenada(atoi(szX), atoi(szY)), atoi(szTipo));
      ColaSecciones.Anadir(seccion);
      Laberinto->Modificar(seccion.Posicion(), seccion.Tipo());
   }
   datos.getline(cad, 20); // Direccion
   direccion = deseada = (eDireccion)atoi(cad); 
   datos.getline(cad, 20); // Por crecer
   porCrecer = atoi(cad);
   Comida->Situar(Laberinto->ObtenerLibre());
}

void CSerpiente::Mostrar(HDC dc)
{
   CSeccion sec;
   
   if(ColaSecciones.Consultar(sec)) 
      do {
         Graficos->Mostrar(sec.Posicion(), sec.Tipo(), dc);
      } while(ColaSecciones.Siguiente(sec));
}

bool CSerpiente::Avanzar()
{
   CCoordenada proxima;
   CSeccion cola;
   int comida = 0;
   int previo;

   // Si el movimiento es ilegal, ignorarlo:
   if(deseada == !direccion) deseada = direccion;
   
   // Tipo de seccin que debe ser la anterior a la nueva cabeza:
   CSeccion &cabeza = ColaSecciones.ObtenerUltimo();
   previo = cabeza.Tipo() & 0x10;
   cabeza.ModificarTipo(previo | (int)!direccion | (int)deseada);  

   // Calcular la nueva posicin de la cabeza:
   proxima = CCoordenada(cabeza.Posicion());
   proxima.Avanzar((eDireccion)(int)deseada);
         
   // Comprobar si hay choque:
   if(Laberinto->Colision(proxima)) // Choque.
      return false;

   // Seccin anterior a cabeza:
   Graficos->Mostrar(cabeza.Posicion(), cabeza.Tipo());

   // Comprobar si hay comida:
   if(proxima == Comida->Posicion()) {
      comida = 0x10;
      porCrecer += CRECER;
      // Nueva comida
      Comida->Situar(Laberinto->ObtenerLibre());
      Comida->Mostrar();
   }

   // Comprobar si hay comida:
   if(Extra->Activo() && proxima == Extra->Posicion()) {
      comida = 0x10;
      porCrecer += 16-Extra->Valor(); // Depende de estado...
      // Eliminar Extra
      Extra->Anular();
   }

   // Aadir nueva cabeza:
   ColaSecciones.Anadir(CSeccion(proxima, comida | (int)deseada));
   Laberinto->Modificar(proxima, comida | (int)deseada);

   // Mostrar nueva cabeza:
   Graficos->Mostrar(proxima, comida | (int)deseada);

   // Borrar cola, si es necesario:
   if(!porCrecer) {
      // Elimina la cola de la cola:
      cola = ColaSecciones.Leer();
      Graficos->Mostrar(cola.Posicion(), LIMPIA);
      // Convertir ltimo actual en cola:
      CSeccion &cola2 = ColaSecciones.ObtenerPrimero();
      cola2.ModificarTipo(CalcularCola(cola.Tipo(), cola2.Tipo()));
      // Actualizar pantalla: 
      Graficos->Mostrar(cola2.Posicion(), cola2.Tipo());
      Laberinto->Modificar(cola.Posicion(), LIMPIA);
   }   
   else {
      Tanteo->Actualizar(1);
      porCrecer--;
   }

   direccion = deseada;
   return true;
}

int CSerpiente::CalcularCola(int anterior, int ultimo)
{
   int valor;

   // Obtener la parte comn:
   valor = anterior & ultimo; // AND
   // Calcular el sentido contrario:
   if(valor < 4) valor <<= 2; else valor >>= 2;
   // Complementar a 2, slo los cuatro ltimos bits:
   valor = 0x0f - valor;
   if(ultimo & 0x10) valor |= 0x10;
   return valor;
}

